// ===========================================================================
// CONTENT  : CLASS DependencySet
// AUTHOR   : M.Duchrow
// VERSION  : 1.1 - 09/02/2007
// HISTORY  :
//  12/01/2007  mdu  CREATED
//	09/02/2007	mdu		changed	-->	Generic type declaration
//
// Copyright (c) 2007, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.odem ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * A set of IDependency objects. It allows to simply lookup particular elements
 * and to iterate over the elements.
 * <p/>
 * Type S defines the type of the source elements in the dependencies supported 
 * by this dependency set and type T specifies the type of the target elements.
 * <br/>
 * Be aware that this implies the assumption that all dependency objects in 
 * a dependency set are of the same type.
 *
 * @author M.Duchrow
 * @version 1.1
 */
public class DependencySet<S extends IExplorableElement,T extends IExplorableElement>
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
	private List<IDependency<S,T>> dependencies ;
	
	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
  /**
   * Initialize the new instance with a default capcity of 50.
   */
  public DependencySet()
  {
    this(50) ;
  } // DependencySet() 

  // -------------------------------------------------------------------------
  
  /**
   * Initialize the new instance with an initial capacity.
   */
  public DependencySet( int initialCapacity )
  {
  	super() ;
  	dependencies = this.newList( initialCapacity ) ;
  } // DependencySet() 
  
  // -------------------------------------------------------------------------
  
  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Add a new dependency.
   */
  public void add( IDependency<S,T> dependency ) 
	{
		dependencies.add( dependency ) ;
	} // add() 
	
	// -------------------------------------------------------------------------

  /**
   * Returns the first dependency with the source element equal to the
   * given element.
   * 
   * @param source The source element to look for
   * @return The dependency or null if not found
   */
  public IDependency<S,T> lookupSourceElement( S source ) 
	{
		return this.findFirstDependencyWith( source, true ) ;
	} // lookupSourceElement() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Returns the first dependency with the a target element equal to the
   * given element.
   * 
   * @param target The target element to look for
   * @return The dependency or null if not found
   */
  public IDependency<S,T> lookupTargetElement( T target ) 
  {
  	return this.findFirstDependencyWith( target, true ) ;
  } // lookupTargetElement() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Adds all dependencies that match the given filter to the provided collection.
   * 
   * @param collection The collection to which the dependencies will be added
   * @param filter The filter that determines which dependencies to add
   */
  public void collectInto( Collection<IDependency<S,T>> collection, IDependencyFilter<IDependency<S,T>> filter ) 
	{
  	if ( ( collection == null ) || ( filter == null ) )
		{
  		return ;
		}
		for (IDependency<S,T> dependency : dependencies )
		{
			if ( filter.matches( dependency ) )
			{
				collection.add( dependency ) ;
			}
		}
	} // collectInto() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Return a list of all dependencies that match the given filter.
   * 
   * @param filter The filter the dependencies must match.
   * @return Always a list, never null
   */
  public List<IDependency<S,T>> collect( IDependencyFilter<IDependency<S,T>> filter ) 
	{
  	List<IDependency<S,T>> result ;
  	
  	result = this.newList( dependencies.size() ) ;
  	this.collectInto( result, filter ) ;
  	return result ;
	} // collect() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Return a list of all dependencies with the given classification.
   * 
   * @param classification The classification of the dependencies to collect.
   * @return Always a list, never null
   */
  public List<IDependency<S,T>> collectForClassification( final DependencyClassification classification ) 
  {
  	IDependencyFilter<IDependency<S,T>> filter ; 
  	
  	filter = new IDependencyFilter<IDependency<S,T>>()
  	{
  		public boolean matches( IDependency<S,T> dependency )
  		{
  			return dependency.getDependencyClassification() == classification ;
  		}
  	} ;
  	return this.collect( filter ) ;
  } // collectForClassification() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns a copy of this dependency set that contains the identical 
   * dependency objects (i.e. no deep copy).
   */
  public DependencySet<S,T> copy() 
	{
  	DependencySet<S,T> copy ;
  	
  	copy = new DependencySet<S, T>( dependencies.size() ) ;
  	copy.addAll( dependencies ) ;
  	return copy ;
	} // copy() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Returns a new list to hold dependency objects of the correct type.
   * Here this methods creates a new java.util.ArrayList.
   * <br/>
   * Subclasses may override this method in order to provide a different
   * type of List implementation.
   * <br/>
   * Whenever a new list is needed in this class then this method will be 
   * used to get one.
   * 
   * @param initialCapacity The initial capacity of the new list.
   */
  public List<IDependency<S, T>> newList( int initialCapacity ) 
	{
		return new ArrayList<IDependency<S,T>>( initialCapacity ) ;
	} // newList() 
	
	// -------------------------------------------------------------------------
  
  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  /**
   * Returns the first dependency with the a source element equal to the
   * given element.
   * 
   * @param element The element to look for
   * @param elementIsSource if true the given element is the source element, otherwise false
   * @return The dependency or null if not found
   */
  protected IDependency<S,T> findFirstDependencyWith( IExplorableElement element, boolean elementIsSource ) 
	{
  	boolean found ;
  	
  	if ( element != null )
		{
  		for (IDependency<S,T> dependency : dependencies )
  		{
  			found = element.equals( elementIsSource ? dependency.getSourceElement() : dependency.getTargetElement() ) ;
  			if ( found )
  			{
  				return dependency ;
  			}
  		}
		}
		return null ;
	} // findFirstDependencyWith() 
	
	// -------------------------------------------------------------------------
  
  /**
   * Adds all the dependencies in the given collection to the internal 
   * dependency list.
   * 
   * @param someDependencies A collection of dependencies (must not be null)
   */
  protected void addAll( Collection<IDependency<S,T>> someDependencies ) 
	{
		dependencies.addAll( someDependencies ) ;
	} // addAll() 
	
	// -------------------------------------------------------------------------
  
} // class DependencySet 
